      subroutine json_xsect(ntitle, title, s02, erelax,
     -       wp, edge, emu,
     1       gamach, ne, ne1, ik0,
     2       col1, col2, col3, col4, col5)

      include '../HEADERS/dim.h'
      integer ntitle, ne, ne1, ik0
      character*80 title(nheadx)
      double precision s02, erelax, wp, edge, emu, gamach
      double precision col1(nex), col2(nex), col3(nex)
      double precision col4(nex), col5(nex)

      return
      end
