c-*-fortran-*-
c     const.h
      double precision pi
      integer one, zero
      double precision third
      double precision raddeg
      double precision fa
      double precision bohr, ryd
      double precision hart
      double precision alpinv
      double precision alphfs

c     dim.h
      integer nclusx
      integer nclxtd
      integer nspx
      integer natx
      integer nattx
      integer lx
      integer nphx
      integer ltot
      integer nrptx
      integer nex, lamtot
      integer mtot, ntot
      integer npatx
      integer legtot
      integer novrx
      integer nheadx
      integer MxPole
