c -*-fortran-*-
      character*20 vfeff
c                       12345678901234567890  
      parameter (vfeff='Feff8L (EXAFS)')

      character*10 vf85e
c                       1234567890
      parameter (vf85e=' 0.1')
