c{vers.h -*-fortran-*-
      character*16  vfeff, vpotph, vpaths, vgenfm, vff2ch

      parameter(vfeff  = 'Feff 6L.02')
      parameter(vpotph = 'potph 4.12')
      parameter(vpaths = 'paths 3.05')
      parameter(vgenfm = 'genfmt 1.44')
      parameter(vff2ch = 'ff2chi 2.01')

c}
